<?php

namespace Litecms\Legal\Http\Controllers;

use Exception;
use Litecms\Legal\Actions\LegalWorkflow;
use Litecms\Legal\Http\Requests\LegalWorkflowRequest;
use Litecms\Legal\Http\Resources\LegalResource;
use Litecms\Legal\Models\Legal;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class LegalWorkflowController extends BaseController
{
    /**
     * Action controller function for legal.
     *
     * @param Model $legal
     * @param action next action for the legal.
     *
     * @return Response
     */
    public function __invoke(LegalWorkflowRequest $request, Legal $legal, $trasition)
    {
        try {
            $request = $request->all();
            $legal = LegalWorkflow::run($trasition, $legal, $request);
            $data = new LegalResource($legal);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('legal::legal.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('legal/legal/' . $legal->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('legal/legal/' . $legal->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the legal and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $legal
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $legal, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $legal = Legal::findBySignedId($legal);
        $transitions = $this->transitions($legal, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('legal::legal.name') . ' ' . trans('Approvals'))
            ->view('legal::public.legal.workflow')
            ->layout('mobile')
            ->data(compact('legal', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the legal.
     *
     * @param PublicRequest $request
     * @param string $legal
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $legal, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $legal = Legal::findBySignedId($legal);
        $user = user();
        $legal = LegalWorkflow::run($transition, $legal, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($legal, $user)
    {
        $transitions = [];
        foreach ($legal->workflow()->transitions($legal) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.legal.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'legal' => $legal->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $legal->workflow()->form($value);
            $array['label'] = trans('legal::legal.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
